#!/usr/bin/perl
# -w
#
# ABSOLUTELY NO WARRANTY WITH THIS PACKAGE. USE IT AT YOUR OWN RISK.
#
#
# Purpose: Lookup info for ID3 tags using MusicBrainz and FreeDB databases
#
#
# Usage: id3butler.pl [options]
#        -a text  artist
#        -l       list ID3 v1 and v2 tags for specified files
#        -r num   list hits with relevance >= num (default: $MinRelevance)
#        -t text  title
#        -y num   year
#        -h       help
#        -v       verbose
#
# Copyright (c) 2005  id3butler.pl 1.00  Iain Lea  iain@bricbrac.de
#
#   This program is free software; you can redistribute it and/or 
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation; either version 2 of
#   the License, or (at your option) any later version.
# 
#   This program is distributed in the hope that it will be useful, 
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
#   See the GNU General Public License for more details.
# 
#   You should have received a copy of the GNU GPL along with this
#   program; if not, write to the Free Software Foundation, Inc.,
#   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#
# Requirements:
#   The following perl modules available from CPAN are required:
#   perl -MCPAN -e shell;
#     install MusicBrainz::Client
#     install CDDB
#     install MP3::Tag
#
#   Remember to make the script executable  'chmod u+x id3butler.pl'
#
#
# Contributors:
#   - 
#
#
# Changelog:
#   1.00
#   - 
#
#
# Todo:
#   - lookup year and genre in FreeDB ?
#   - how to calculate the time field for MusicBrainz ?

#use strict;
require 'getopts.pl';
use MusicBrainz::Client;
use MusicBrainz::Queries qw(:all);
use CDDB;
use MP3::Tag;
  
# constants
my $Version = 'v1.00';
my $ScriptName = 'id3butler.pl';
my $ScriptUrl = "http://iainlea.dyndns.org/software/mp3butler";
my $Copyright = "Copyright 2005 (c)  Iain Lea  iain\@bricbrac.de";
#
# variables
my $Id3TagArtist = "";
my $Id3TagTitle = "";
my $Id3TagYear = -1;
my $MinRelevance = 80;
my $Verbose = 0;
#
my $Result;
my $Query;
my $Type;
my $Frag;
my $Mb;

$| = 1;

&ParseCmdLine ($0, 0);
        
$Mb = MusicBrainz::Client->new ();

$Query = [ "", "$Id3TagArtist", "", "$Id3TagTitle" ];

if (! $Mb->query_with_args (MBQ_FileInfoLookup, $Query)) {
  printf "Error: query_with_args - %s\n", $Mb->get_query_error();
  exit 1;
}
                  
if (! $Mb->select1 (MBS_SelectLookupResult, 1)) {
  print "Error: select1 - No lookup result\n";
  exit 1;
}

$Type = $Mb->get_result_data (MBE_LookupGetType);
$Frag = $Mb->get_fragment_from_url ($Type);
    
if ($Frag eq 'AlbumTrackResult') {
  ($Result, $Index) = &HandleAlbumTrackList ($Mb);

#use Data::Dumper;
#print Dumper($Result);

  printf "# relevance, artist, track, album, track count, time\n" if $Index - 1;

  foreach $Num (0 .. $Index - 1) {
    if ($Result->[$Num]->{relevance} >= $MinRelevance) {
      printf "%d,%s,%s,%s,%d,%d\n",
        $Result->[$Num]->{relevance},
        &CleanTag ($Result->[$Num]->{artist}),
        &CleanTag ($Result->[$Num]->{track}),
        &CleanTag ($Result->[$Num]->{album}),
        $Result->[$Num]->{total},
        $Result->[$Num]->{time};
    }
  }
} else {
  die "Error: not an AlbumTrackResult; instead of type '$Frag'";
}

exit 0;


#######################################################################
# SUBROUTINES

sub ParseCmdLine
{
	my ($ProgPath, $ShowUsage) = @_;
	my $ProgName = $ProgPath;

	$ProgName = $1 if ($ProgName =~ /.*\/([^\/]*)/);

	&Getopts ('a:hlr:t:vy:');

	if ($opt_h || $ShowUsage) { 
		print <<EOT
$ScriptName $Version  URL: $ScriptUrl

$Copyright

Lookup info for ID3 tags using MusicBrainz and FreeDB databases

Usage: $ProgName [options]
       -a text  artist
       -l       list ID3 v1 and v2 tags for specified files
       -r num   list hits with relevance >= num (default: $MinRelevance)
       -t text  title
       -y num   year
       -h       help
       -v       verbose

Examples:
  $ProgName -a the_artist -t the_title -r 95

    display all matching hits with a relevance of 95 or higher
EOT
;
		exit 1;
	}

	$Id3TagArtist = $opt_a if (defined ($opt_a));
	$Id3TagTitle = $opt_t if (defined ($opt_t));
	$Id3TagYear = $opt_y if (defined ($opt_y));
	$ListId3Tag = 1 if (defined ($opt_l));
	$MinRelevance = $opt_r if (defined ($opt_r));
	$Verbose = 1 if (defined ($opt_v));

#	@FileList = @ARGV;
#	if ($#FileList == -1) {
#		&ParseCmdLine ($0, 1);
#	}

	if (! $Id3TagArtist || ! $Id3TagTitle) {
	    print "Error: -a artist and -t title options are required\n\n";
		&ParseCmdLine ($0, 1);
	}
}


sub HandleAlbumTrackList 
{
  my ($Mb) = @_;
  my $Index, $Result, $Relevance, $Track, $Length, $Artist, $Album;
  my $TrackCount, $Year, $Genre;
          
  for ($Index = 1;; $Index++) {
    $Mb->select (MBS_Rewind);
                    
    if (! $Mb->select1 (MBS_SelectLookupResult, $Index)) {
      last;
    }
      
    $Relevance = $Mb->get_result_int (MBE_LookupGetRelevance);
            
    # get track info
    $Mb->select (MBS_SelectLookupResultTrack);
    $Track  = $Mb->get_result_data (MBE_TrackGetTrackName);
    $Length = $Mb->get_result_data (MBE_TrackGetTrackDuration);     
    $Artist = $Mb->get_result_data (MBE_TrackGetArtistName);
    $Mb->select (MBS_Back);

    # get album info
    $Mb->select (MBS_SelectLookupResultAlbum);
    $Album = $Mb->get_result_data (MBE_AlbumGetAlbumName);
    $Count = $Mb->get_result_int (MBE_AlbumGetNumTracks);

    $Year  = $Mb->get_result_int (MBE_AlbumGetYear);
    $Genre = $Mb->get_result_int (MBE_AlbumGetGenre);

    $Mb->select (MBS_Back);

    $Result->[$Index-1] = { 
      relevance => $Relevance,
      track     => $Track,
      album     => $Album,
      artist    => $Artist,
      total     => $Count,
      time      => $Length,
#      year      => $Year,
#      genre     => $Genre,
    };
}
  
  return ($Result, --$Index);
}


sub CleanTag 
{
  my ($Tag) = @_;

  $Tag =~ s/,/___COMMA___/g;
  
  return $Tag;  
}
